@@ -1,45 +1,41 @@
 /* -----------------------------------------------------------------------------
-The copyright in this software is being made available under the BSD
+The copyright in this software is being made available under the Clear BSD
 License, included below. No patent rights, trademark rights and/or 
 other Intellectual Property Rights other than the copyrights concerning 
 the Software are granted under this license.
 
-For any license concerning other Intellectual Property rights than the software,
-especially patent licenses, a separate Agreement needs to be closed. 
-For more information please contact:
+The Clear BSD License
 
-Fraunhofer Heinrich Hertz Institute
-Einsteinufer 37
-10587 Berlin, Germany
-www.hhi.fraunhofer.de/vvc
-vvc@hhi.fraunhofer.de
-
-Copyright (c) 2019-2021, Fraunhofer-Gesellschaft zur Förderung der angewandten Forschung e.V.
+Copyright (c) 2019-2023, Fraunhofer-Gesellschaft zur Förderung der angewandten Forschung e.V. & The VVenC Authors.
 All rights reserved.
 
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions are met:
+Redistribution and use in source and binary forms, with or without modification,
+are permitted (subject to the limitations in the disclaimer below) provided that
+the following conditions are met:
 
- * Redistributions of source code must retain the above copyright notice,
-   this list of conditions and the following disclaimer.
- * Redistributions in binary form must reproduce the above copyright notice,
-   this list of conditions and the following disclaimer in the documentation
-   and/or other materials provided with the distribution.
- * Neither the name of Fraunhofer nor the names of its contributors may
-   be used to endorse or promote products derived from this software without
-   specific prior written permission.
+     * Redistributions of source code must retain the above copyright notice,
+     this list of conditions and the following disclaimer.
 
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
-BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
-THE POSSIBILITY OF SUCH DAMAGE.
+     * Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
+
+     * Neither the name of the copyright holder nor the names of its
+     contributors may be used to endorse or promote products derived from this
+     software without specific prior written permission.
+
+NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY
+THIS LICENSE. THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
+CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
+BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
+IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+POSSIBILITY OF SUCH DAMAGE.
 
 
 ------------------------------------------------------------------------------------------- */
@@ -48,8 +44,8 @@ THE POSSIBILITY OF SUCH DAMAGE.
   \brief   This file contains the external interface of the vvenc SDK.
 */
 
-#ifndef _VVENC_H_
-#define _VVENC_H_
+#ifndef VVENC_VVENC_H
+#define VVENC_VVENC_H
 
 #include "vvenc/vvencDecl.h"
 
@@ -122,7 +118,7 @@ typedef struct vvencYUVBuffer
 {
   vvencYUVPlane planes[ 3 ];           // plane buffer for 3 components (yuv)
   uint64_t      sequenceNumber;        // sequence number of the picture
-  uint64_t      cts;                   // composition time stamp in TicksPerSecond (see HEVCEncoderParameter)
+  uint64_t      cts;                   // composition time stamp in TicksPerSecond
   bool          ctsValid;              // composition time stamp valid flag (true: valid, false: CTS not set)
 }vvencYUVBuffer;
 
@@ -279,11 +275,12 @@ VVENC_DECL int vvenc_encoder_set_RecYUVBufferCallback(vvencEncoder *, void * ctx
 /* vvenc_init_pass
   This method initializes the encoder instance in dependency to the encoder pass.
  \param[in]  vvencEncoder pointer to opaque handler
- \param[in]  pass number of current pass to init
+ \param[in]  pass number of current pass to init (0: first pass, 1: second pass )
+ \param[in]  rate control statistics file name
  \retval     int if non-zero an error occurred (see ErrorCodes), otherwise VVENC_OK indicates success.
  \pre        None
 */
-VVENC_DECL int vvenc_init_pass( vvencEncoder *, int pass );
+VVENC_DECL int vvenc_init_pass( vvencEncoder *, int pass, const char * statsFName );
 
 /* vvenc_encode
   This method encodes a picture.
@@ -335,6 +332,15 @@ VVENC_DECL int vvenc_reconfig( vvencEncoder *, const vvenc_config * );
 */
 VVENC_DECL int vvenc_check_config( vvencEncoder *, const vvenc_config * );
 
+/* vvenc_get_headers
+ This method returns the headers (SPS,PPS,...) that are used.
+ All init calls (vvenc_encoder_open, vvenc_init_pass) must be called in advance.
+ \param[in]  vvencEncoder pointer to opaque handler
+ \param[out] accessUnit pointer to vvencAccessUnit that retrieves compressed access units containing all headers.    
+ \retval     int negative indicates an error, otherwise the return value VVENC_OK indicates success.
+*/
+VVENC_DECL int vvenc_get_headers(vvencEncoder *, vvencAccessUnit * );
+
 /* vvenc_get_last_error
  This method returns the last occurred error as a string.
  \param[in]  vvencEncoder pointer to opaque handler
@@ -377,9 +383,11 @@ VVENC_DECL int vvenc_print_summary( vvencEncoder * );
 */
 VVENC_DECL const char* vvenc_get_error_msg( int nRet );
 
-/* vvenc_set_logging_callback
- This method registers a log message callback function to the encoder library.
+/* vvenc_set_logging_callback *deprecated*
+ This method registers a global log message callback function to the encoder library.
  If no such function has been registered, the library will omit all messages.
+ *deprecated* - This method is deprecated since it uses a global logger and will be removed in the next major version.
+                Please use the method vvenc_set_logging_callback(vvenc_config,void *,vvencLoggingCallback) to register a thread safe local looger
  \param[in]  ctx pointer of the caller, if not needed set it to null
  \paramin]   Log message callback function.
  \retval     int VVENC_ERR_INITIALIZE indicates the encoder was not successfully initialized in advance, otherwise the return value VVENC_OK indicates success.
@@ -425,10 +433,12 @@ VVENC_DECL int  vvenc_get_height_of_component( const vvencChromaFormat chFmt, co
 VVENC_DECL bool  vvenc_is_tracing_enabled( void );
 
 /* vvenc_decode_bitstream
- * set filename of a bitstream that should be decoded
- \retval[ ] const char* file name of bitstream
+ \param[in]  FileName of bitstream that should be decoded
+ \param[in]  trcFile filename of a trace rule file
+ \param[in]  trcRule trace rules
+ \retval     int VVENC_ERR_INITIALIZE indicates the encoder was not successfully initialized in advance, otherwise the return value VVENC_OK indicates success.
 */
-VVENC_DECL int   vvenc_decode_bitstream( const char* FileName);
+VVENC_DECL int   vvenc_decode_bitstream( const char* FileName, const char* trcFile, const char* trcRule);
 
 
 VVENC_NAMESPACE_END
@@ -437,4 +447,4 @@ VVENC_NAMESPACE_END
 }
 #endif /*__cplusplus */
 
-#endif /*_VVENC_H_*/
+#endif /*VVENC_VVENC_H*/
