@@ -1,45 +1,41 @@
 /* -----------------------------------------------------------------------------
-The copyright in this software is being made available under the BSD
+The copyright in this software is being made available under the Clear BSD
 License, included below. No patent rights, trademark rights and/or 
 other Intellectual Property Rights other than the copyrights concerning 
 the Software are granted under this license.
 
-For any license concerning other Intellectual Property rights than the software,
-especially patent licenses, a separate Agreement needs to be closed. 
-For more information please contact:
+The Clear BSD License
 
-Fraunhofer Heinrich Hertz Institute
-Einsteinufer 37
-10587 Berlin, Germany
-www.hhi.fraunhofer.de/vvc
-vvc@hhi.fraunhofer.de
-
-Copyright (c) 2019-2021, Fraunhofer-Gesellschaft zur Förderung der angewandten Forschung e.V.
+Copyright (c) 2019-2023, Fraunhofer-Gesellschaft zur Förderung der angewandten Forschung e.V. & The VVenC Authors.
 All rights reserved.
 
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions are met:
+Redistribution and use in source and binary forms, with or without modification,
+are permitted (subject to the limitations in the disclaimer below) provided that
+the following conditions are met:
 
- * Redistributions of source code must retain the above copyright notice,
-   this list of conditions and the following disclaimer.
- * Redistributions in binary form must reproduce the above copyright notice,
-   this list of conditions and the following disclaimer in the documentation
-   and/or other materials provided with the distribution.
- * Neither the name of Fraunhofer nor the names of its contributors may
-   be used to endorse or promote products derived from this software without
-   specific prior written permission.
+     * Redistributions of source code must retain the above copyright notice,
+     this list of conditions and the following disclaimer.
 
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
-BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
-THE POSSIBILITY OF SUCH DAMAGE.
+     * Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
+
+     * Neither the name of the copyright holder nor the names of its
+     contributors may be used to endorse or promote products derived from this
+     software without specific prior written permission.
+
+NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY
+THIS LICENSE. THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
+CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
+BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
+IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+POSSIBILITY OF SUCH DAMAGE.
 
 
 ------------------------------------------------------------------------------------------- */
@@ -65,6 +61,17 @@ extern "C" {
 
 VVENC_NAMESPACE_BEGIN
 
+/* vvdecLoggingCallback:
+   callback function to receive messages of the encoder library
+  \param[in]  void* caller contex
+  \param[in]  int log level
+  \param[in]  const char* fmt
+  \param[in]  va_list args
+  \retval     none
+*/
+typedef void (*vvencLoggingCallback)(void*, int, const char*, va_list);
+
+
 
 // ====================================================================================================================
 
@@ -73,11 +80,15 @@ VVENC_NAMESPACE_BEGIN
 #define VVENC_MAX_TLAYER                      7      // Explicit temporal layer QP offset - max number of temporal layer
 #define VVENC_MAX_NUM_CQP_MAPPING_TABLES      3      // Maximum number of chroma QP mapping tables (Cb, Cr and joint Cb-Cr)
 #define VVENC_MAX_NUM_ALF_ALTERNATIVES_CHROMA 8
-#define VVENC_MCTF_RANGE                      2      // max number of frames used for MCTF filtering in forward / backward direction
+#define VVENC_MCTF_RANGE                      4      // max number of frames used for MCTF filtering in forward / backward direction
 #define VVENC_MAX_NUM_COMP                    3      // max number of components
 #define VVENC_MAX_QP_VALS_CHROMA              8      // max number qp vals in array
 #define VVENC_MAX_MCTF_FRAMES                 16
 #define VVENC_MAX_STRING_LEN                  1024   // max length of string/filename
+#define VVENC_DEFAULT_QP                      32     // default base QP
+#define VVENC_AUTO_QP                        -1      // indicates to use default QP, or ignore if RC is used
+#define VVENC_RC_OFF                          0      // indicates rate control is disabled
+#define VVENC_TICKS_PER_SEC_DEF               27000000 // default ticks per second
 
 // ====================================================================================================================
 
@@ -183,7 +194,9 @@ typedef enum
   VVENC_DRT_NONE               = 0,
   VVENC_DRT_CRA                = 1,
   VVENC_DRT_IDR                = 2,
-  VVENC_DRT_RECOVERY_POINT_SEI = 3
+  VVENC_DRT_RECOVERY_POINT_SEI = 3,
+  VVENC_DRT_IDR2               = 4,
+  VVENC_DRT_CRA_CRE            = 5,             //constrained RASL encoding
 }vvencDecodingRefreshType;
 
 typedef enum
@@ -197,12 +210,15 @@ typedef enum
 
 typedef enum
 {
-  VVENC_HDR_OFF     = 0, // SDR
+  VVENC_HDR_OFF     = 0, // undefined - no signalization
   VVENC_HDR_PQ,          // HDR10, Dolby
   VVENC_HDR_HLG,         // Hybrid Log Gamma
   VVENC_HDR_PQ_BT2020,   // HDR10, Dolby + BT.2020
   VVENC_HDR_HLG_BT2020,  // Hybrid Log Gamma + BT.2020
-  VVENC_HDR_USER_DEFINED // user defined HDR mode (to provide old HDR modes, HDR is set individually)
+  VVENC_HDR_USER_DEFINED,// user defined HDR mode (to provide old HDR modes, HDR is set individually)
+  VVENC_SDR_BT709,       // SDR BT.709 
+  VVENC_SDR_BT2020,      // SDR BT.2020
+  VVENC_SDR_BT470BG      // SDR BT.470 B/G
 }vvencHDRMode;
 
 typedef enum
@@ -258,16 +274,7 @@ typedef enum
 
 typedef enum
 {
-  VVENC_RCM_AUTO          = -1,
-  VVENC_RCM_OFF           = 0,
-  VVENC_RCM_CTU_LEVEL     = 1,
-  VVENC_RCM_PICTURE_LEVEL = 2,
-  VVENC_RCM_GOP_LEVEL     = 3
-}vvencRateControlMode;
-
-typedef enum
-{
-  VVENC_FASTINTERSEARCH_AUTO     = 0,
+  VVENC_FASTINTERSEARCH_OFF      = 0,
   VVENC_FASTINTERSEARCH_MODE1    = 1,
   VVENC_FASTINTERSEARCH_MODE2    = 2,
   VVENC_FASTINTERSEARCH_MODE3    = 3
@@ -278,7 +285,7 @@ typedef enum
 {
   VVENC_MESEARCH_FULL              = 0,
   VVENC_MESEARCH_DIAMOND           = 1,
-  VVENC_MESEARCH_SELECTIVE         = 2,
+  VVENC_MESEARCH_DEPRECATED        = 2,
   VVENC_MESEARCH_DIAMOND_ENHANCED  = 3,
   VVENC_MESEARCH_DIAMOND_FAST      = 4,
   VVENC_MESEARCH_NUMBER_OF_METHODS = 5
@@ -314,18 +321,18 @@ typedef struct vvencGOPEntry
   double m_QPFactor;
   int    m_tcOffsetDiv2;
   int    m_betaOffsetDiv2;
-  int    m_CbTcOffsetDiv2;
-  int    m_CbBetaOffsetDiv2;
-  int    m_CrTcOffsetDiv2;
-  int    m_CrBetaOffsetDiv2;
+  int    m_cfgUnused1;                                                                   // TODO: remove unused memory from configuration
+  int    m_cfgUnused2;
+  int    m_cfgUnused3;
+  int    m_cfgUnused4;
   int    m_temporalId;
-  bool   m_refPic;
+  bool   m_cfgUnused5;                                                                   // TODO: remove unused memory from configuration
   char   m_sliceType;
   int    m_numRefPicsActive[ 2 ];
   int    m_numRefPics[ 2 ];
   int    m_deltaRefPics[ 2 ][ VVENC_MAX_NUM_REF_PICS ];
-  bool   m_isEncoded;
-  bool   m_ltrp_in_slice_header_flag;
+  bool   m_cfgUnused6;                                                                   // TODO: remove unused memory from configuration
+  bool   m_cfgUnused7;
 }vvencGOPEntry;
 
 VVENC_DECL void vvenc_GOPEntry_default(vvencGOPEntry *GOPEntry );
@@ -347,7 +354,9 @@ typedef struct vvencRPLEntry
 
 VVENC_DECL void vvenc_RPLEntry_default(vvencRPLEntry *RPLEntry );
 
-
+/// <summary>
+/// Deprecated, dont use!
+/// </summary>
 typedef struct vvencWCGChromaQPControl
 {
   bool   enabled        ;    // Enabled flag (0:default)
@@ -381,8 +390,8 @@ typedef struct vvencReshapeCW
   int          adpOption;
   unsigned int initialCW;
   int          rspPicSize;
+  int          rspUnused;
   int          rspFps;
-  int          rspBaseQP;
   int          rspTid;
   int          rspFpsToIp;
 }vvencReshapeCW;
@@ -395,8 +404,8 @@ typedef struct vvencMCTF
   int                 MCTF;
   int                 MCTFSpeed;
   bool                MCTFFutureReference;
-  int                 MCTFNumLeadFrames;
-  int                 MCTFNumTrailFrames;
+  int                 MCTFUnitSize;
+  int                 mctfUnused2;                                                       // TODO: remove unused memory from configuration
 
   int                 numFrames;
   int                 MCTFFrames[VVENC_MAX_MCTF_FRAMES];
@@ -410,43 +419,46 @@ VVENC_DECL void vvenc_vvencMCTF_default(vvencMCTF *vvencMCTF );
 
 typedef struct vvenc_config
 {
-
-  // basic config params
-  bool                m_configDone;
-  bool                m_confirmFailed;                                                   // state variable
-
-  vvencMsgLevel       m_verbosity;                                                       // encoder verbosity
-  int                 m_framesToBeEncoded;                                               // number of encoded frames
-
-  int                 m_FrameRate;                                                       // source frame-rates (Hz)
-  int                 m_FrameSkip;                                                       // number of skipped frames from the beginning
+  // core config params
   int                 m_SourceWidth;                                                     // source width in pixel
   int                 m_SourceHeight;                                                    // source height in pixel (when interlaced = field height)
-  int                 m_TicksPerSecond;                                                  // ticks per second e.g. 90000 for dts generation (1..27000000)
+  int                 m_FrameRate;                                                       // source frame-rates (Hz) Numerator
+  int                 m_FrameScale;                                                      // source frame-rates (Hz) Denominator
+  int                 m_TicksPerSecond;                                                  // ticks per second for dts generation (default: 27000000, 1..27000000, -1: ticks per frame=1)
+  int                 m_framesToBeEncoded;                                               // number of encoded frames (default: 0, all)
+  int                 m_inputBitDepth[ 2 ];                                              // bit-depth of input pictures (2d array for luma,chroma)
 
-  vvencProfile        m_profile;
-  vvencTier           m_levelTier;
-  vvencLevel          m_level;
+  int                 m_numThreads;                                                      // number of worker threads ( if <0: <720p 4threads, else 8threads (limited to available cores))
 
-  int                 m_IntraPeriod;                                                     // period of I-slice (random access period)
+  int                 m_QP;                                                              // QP value of key-picture (0-63, default: 32)
+  int                 m_RCTargetBitrate;                                                 // target bitrate in bps, (default. 0 (rc disabled))
+
+  vvencMsgLevel       m_verbosity;                                                       // encoder verbosity level
+
+  // basic config params
+
+  vvencProfile        m_profile;                                                         // profile 
+  vvencTier           m_levelTier;                                                       // Tier to use for interpretation of level (main or high)
+  vvencLevel          m_level;                                                           // level limit
+
+  int                 m_IntraPeriod;                                                     // period of I-slice in frames  (random access period)
   int                 m_IntraPeriodSec;                                                  // period of I-slice in seconds (random access period)
   vvencDecodingRefreshType m_DecodingRefreshType;                                        // random access type
-  int                 m_GOPSize;                                                          // GOP size of hierarchical structure
+  int                 m_GOPSize;                                                         // GOP size of hierarchical structure
 
-  int                 m_QP;                                                              // QP value of key-picture (integer)
-  bool                m_usePerceptQPA;                                                   // Mode of perceptually motivated input-adaptive QP modification, abbrev. perceptual QP adaptation (QPA).
+  int                 m_RCNumPasses;                                                     // number of rc passes (default: -1, if not set and bitrate > 0 2-pass rc will be used)
+  int                 m_RCPass;                                                          // current pass (0,1) for rc (only needed for 2-pass rc) 
+  bool                m_cfgUnused1;                                                      // TODO: remove unused memory from configuration
 
-  int                 m_RCTargetBitrate;
-  int                 m_RCNumPasses;
+  int                 m_internalBitDepth[ 2 ];                                           // bit-depth codec operates at (input/output files will be converted) (2d array for luma,chroma)
 
-  vvencSegmentMode    m_SegmentMode;
+  vvencHDRMode        m_HdrMode;                                                         // High Dynamic Range mode (default: off)
+  vvencSegmentMode    m_SegmentMode;                                                     // segment position for segment concatenation (only needed, when multiple separate segments are used)
 
-  int                 m_numThreads;                                                      // number of worker threads
+  bool                m_usePerceptQPA;                                                   // usage of perceptually motivated input-adaptive QP modification, abbrev. perceptual QP adaptation (QPA).
 
-  int                 m_inputBitDepth   [ 2 ];                                           // bit-depth of input file
-  int                 m_internalBitDepth[ 2 ];                                           // bit-depth codec operates at (input/output files will be converted)
-
-  vvencHDRMode        m_HdrMode;
+  uint32_t            m_numTileCols;                                                     // number of tile columns
+  uint32_t            m_numTileRows;                                                     // number of tile rows
 
   // expert config params
   int                 m_conformanceWindowMode;
@@ -455,7 +467,7 @@ typedef struct vvenc_config
   int                 m_confWinTop;
   int                 m_confWinBottom;
 
-  unsigned            m_temporalSubsampleRatio;                                          // temporal subsample ratio, 2 means code every two frames
+  unsigned            m_cfgUnused15;
 
   int                 m_PadSourceWidth;                                                  // source width in pixel
   int                 m_PadSourceHeight;                                                 // source height in pixel (when interlaced = field height)
@@ -474,17 +486,19 @@ typedef struct vvenc_config
   unsigned            m_bitDepthConstraintValue;
   bool                m_intraOnlyConstraintFlag;
 
-  int                 m_InputQueueSize;                                                  // Size of frame input queue
   bool                m_rewriteParamSets;                                                // Flag to enable rewriting of parameter sets at random access points
   bool                m_idrRefParamList;                                                 // indicates if reference picture list syntax elements are present in slice headers of IDR pictures
-  vvencRPLEntry       m_RPLList0[ VVENC_MAX_GOP ];                                       // the RPL entries from the config file
-  vvencRPLEntry       m_RPLList1[ VVENC_MAX_GOP ];                                       // the RPL entries from the config file
-  vvencGOPEntry       m_GOPList [ VVENC_MAX_GOP ];                                       // the coding structure entries from the config file
-  int                 m_maxDecPicBuffering[ VVENC_MAX_TLAYER ];                          // total number of pictures in the decoded picture buffer
-  int                 m_maxNumReorderPics [ VVENC_MAX_TLAYER ];                          // total number of reorder pictures
-  int                 m_maxTempLayer;                                                    // Max temporal layer
-  int                 m_numRPLList0;
-  int                 m_numRPLList1;
+
+  vvencRPLEntry       m_cfgUnused2[ 64 ];                                                // TODO: remove unused memory from configuration
+  vvencRPLEntry       m_cfgUnused3[ 64 ];
+
+  vvencGOPEntry       m_GOPList[ VVENC_MAX_GOP ];                                        // the coding structure entries from the config file
+
+  int                 m_cfgUnused4[ 7 ];                                                 // TODO: remove unused memory from configuration
+  int                 m_cfgUnused5[ 7 ];
+  int                 m_cfgUnused6;
+  int                 m_cfgUnused7;
+  int                 m_cfgUnused8;
 
   bool                m_useSameChromaQPTables;
   vvencChromaQpMappingTableParams m_chromaQpMappingTableParams;
@@ -533,12 +547,12 @@ typedef struct vvenc_config
   unsigned            m_MinQT[ 3 ];                                                      // 0: I slice luma; 1: P/B slice; 2: I slice chroma
   unsigned            m_maxMTTDepth;
   unsigned            m_maxMTTDepthI;
-  unsigned            m_maxMTTDepthIChroma;
+  int                 m_maxMTTDepthIChroma;
   unsigned            m_maxBT[3];
   unsigned            m_maxTT[3];
   bool                m_dualITree;
-  unsigned            m_MaxCodingDepth;                                                  // max. total CU depth - includes depth of transform-block structure
-  unsigned            m_log2DiffMaxMinCodingBlockSize;                                   // difference between largest and smallest CU depth
+  unsigned            m_cfgUnused9;                                                      // TODO: remove unused memory from configuration
+  unsigned            m_cfgUnused10;
   int                 m_log2MaxTbSize;
   int                 m_log2MinCodingBlockSize;
 
@@ -546,22 +560,22 @@ typedef struct vvenc_config
   bool                m_bUseHADME;                                                       // flag for using HAD in sub-pel ME
   int                 m_RDOQ;                                                            // flag for using RD optimized quantization
   bool                m_useRDOQTS;                                                       // flag for using RD optimized quantization for transform skip
-  bool                m_useSelectiveRDOQ;                                                // flag for using selective RDOQ
+  int8_t              m_useSelectiveRDOQ;                                                // flag for using selective RDOQ
 
   bool                m_JointCbCrMode;
   int                 m_cabacInitPresent;
   bool                m_useFastLCTU;
-  bool                m_usePbIntraFast;
+  int                 m_usePbIntraFast;
   int                 m_useFastMrg;
   int                 m_useAMaxBT;
   bool                m_fastQtBtEnc;
   bool                m_contentBasedFastQtbt;
   int                 m_fastInterSearchMode;                                             // Parameter that controls fast encoder settings
-  bool                m_bUseEarlyCU;                                                     // flag for using Early CU setting
+  int                 m_useEarlyCU;                                                      // flag for using Early CU setting
   bool                m_useFastDecisionForMerge;                                         // flag for using Fast Decision Merge RD-Cost
 
   bool                m_bDisableIntraCUsInInterSlices;                                   // Flag for disabling intra predicted CUs in inter slices.
-  bool                m_bUseConstrainedIntraPred;                                        // flag for using constrained intra prediction
+  bool                m_cfgUnused11;                                                     // TODO: remove unused memory from configuration
   bool                m_bFastUDIUseMPMEnabled;
   bool                m_bFastMEForGenBLowDelayEnabled;
 
@@ -581,11 +595,11 @@ typedef struct vvenc_config
   int                 m_IntraEstDecBit;                                                  // Intra estimation decimation factor.
 
   int                 m_RCInitialQP;
-  bool                m_RCForceIntraQP;
+  bool                m_cfgUnused16;                                                     // TODO: remove unused memory from configuration
 
   int                 m_motionEstimationSearchMethod;
   int                 m_motionEstimationSearchMethodSCC;
-  bool                m_bRestrictMESampling;                                             // Restrict sampling for the Selective ME
+  bool                m_cfgUnused12;                                                     // TODO: remove unused memory from configuration
   int                 m_SearchRange;                                                     // ME search range
   int                 m_bipredSearchRange;                                               // ME search range for bipred refinement
   int                 m_minSearchWindow;                                                 // ME minimum search window size for the Adaptive Window ME
@@ -644,11 +658,10 @@ typedef struct vvenc_config
   bool                m_loopFilterOffsetInPPS;                                           // offset for deblocking filter in 0 = slice header, 1 = PPS
   int                 m_loopFilterBetaOffsetDiv2[3];                                     // beta offset for deblocking filter
   int                 m_loopFilterTcOffsetDiv2[3];                                       // tc offset for deblocking filter
-  int                 m_deblockingFilterMetric;
+  int                 m_cfgUnused13;                                                     // TODO: remove unused memory from configuration
 
-  bool                m_bLFCrossTileBoundaryFlag;
-  bool                m_bLFCrossSliceBoundaryFlag;                                       // 1: filter across slice boundaries 0: do not filter across slice boundaries
-  bool                m_loopFilterAcrossSlicesEnabled;
+  bool                m_bDisableLFCrossTileBoundaryFlag;                                 // 0: filter across tile boundaries 1: do not filter across tile boundaries
+  bool                m_bDisableLFCrossSliceBoundaryFlag;                                // 0: filter across slice boundaries 1: do not filter across slice boundaries
 
   bool                m_bUseSAO;
   double              m_saoEncodingRate;                                                 // When >0 SAO early picture termination is enabled for luma and chroma
@@ -658,7 +671,7 @@ typedef struct vvenc_config
 
   bool                m_decodingParameterSetEnabled;                                     // enable decoding parameter set
   int                 m_vuiParametersPresent;                                            // enable generation of VUI parameters; -1 auto enable, 0: off 1: enable
-  int                 m_hrdParametersPresent;                                            // enable generation or HRD parameters; -1 auto enable, 0: off 1: enable
+  bool                m_hrdParametersPresent;                                            // enable generation or HRD parameters; 0: off 1: enable
   bool                m_aspectRatioInfoPresent;                                          // Signals whether aspect_ratio_idc is present
   int                 m_aspectRatioIdc;                                                  // aspect_ratio_idc
   int                 m_sarWidth;                                                        // horizontal size of the sample aspect ratio
@@ -673,7 +686,7 @@ typedef struct vvenc_config
   int                 m_chromaSampleLocType;                                             // Specifies the location of chroma samples for progressive content
   bool                m_overscanInfoPresent;                                             // Signals whether overscan_appropriate_flag is present
   bool                m_overscanAppropriateFlag;                                         // Indicates whether conformant decoded pictures are suitable for display using overscan
-  bool                m_videoSignalTypePresent;                                          // Signals whether video_format, video_full_range_flag, and colour_description_present_flag are present
+  bool                m_cfgUnused14;                                                     // TODO: remove unused memory from configuration
   bool                m_videoFullRangeFlag;                                              // Indicates the black level and range of luma and chroma signals
 
   unsigned int        m_masteringDisplay[10];                                            // mastering display colour volume, vector of size 10, format: G(x,y)B(x,y)R(x,y)WP(x,y)L(max,min), 0 <= GBR,WP <= 50000, 0 <= L <= uint (SEI)
@@ -693,15 +706,24 @@ typedef struct vvenc_config
 
   vvencMCTF           m_vvencMCTF;
 
-  int                 m_dqThresholdVal;
+  int                 m_quantThresholdVal;
   int                 m_qtbttSpeedUp;
+  int                 m_qtbttSpeedUpMode;
+  int                 m_fastTTSplit;
+  float               m_fastTT_th;
 
   int                 m_fastLocalDualTreeMode;
 
   int                 m_maxParallelFrames;
   int                 m_ensureWppBitEqual;                                               // Flag indicating bit equalitiy for single thread runs respecting multithread restrictions
+  bool                m_tileParallelCtuEnc;
 
   bool                m_picPartitionFlag;
+  unsigned int        m_tileColumnWidth[10];
+  unsigned int        m_tileRowHeight[10];
+  uint32_t            m_numExpTileCols;                                                  // number of explicitly specified tile columns
+  uint32_t            m_numExpTileRows;                                                  // number of explicitly specified tile rows
+  uint32_t            m_numSlicesInPic;                                                  // derived number of rectangular slices in the picture (raster-scan slice specified at slice level)
 
   // decode bitstream options
   int                 m_switchPOC;                                                       // dbg poc.
@@ -720,16 +742,168 @@ typedef struct vvenc_config
   char                m_summaryOutFilename[VVENC_MAX_STRING_LEN];                        // filename to use for producing summary output file.
   char                m_summaryPicFilenameBase[VVENC_MAX_STRING_LEN];                    // Base filename to use for producing summary picture output files. The actual filenames used will have I.txt, P.txt and B.txt appended.
   unsigned            m_summaryVerboseness;                                              // Specifies the level of the verboseness of the text output.
+  int                 m_numIntraModesFullRD;                                             // Number Modes for Full RD Intra Search
+  bool                m_reduceIntraChromaModesFullRD;                                    // Reduce Number Modes for Full RD Intra Chroma Search
+
+  int                 m_FirstPassMode;
+  int                 m_numRefPics;                                                      // Number of reference pictures
+  int                 m_numRefPicsSCC;                                                   // Number of reference pictures
+  int                 m_alfUnitSize;                                                     // Size of the Alf Search Unit
+  int                 m_meReduceTap;                                                     // Reduce filter taps during motion search (0 - don't, use full 8-tap fitler; 1 - use 6-tap fitlers; 2 - use chroma 4-tap filters)
+  int                 m_deblockLastTLayers;
+  int                 m_leadFrames;                                                      // number of leading frames to to be given before starting with POC 0
+  int                 m_trailFrames;                                                     // number of trailing frames to to be given after last frame to be encoded
+  int                 m_LookAhead;                                                       // enable pre-analysis pass with picture look-ahead
+  int                 m_explicitAPSid;
+
+  bool                m_picReordering;
+  bool                m_reservedFlag[3];
+  bool                m_blockImportanceMapping;
+  bool                m_saoScc;
+  bool                m_addGOP32refPics;
+  bool                m_fastHad;
+  int8_t              m_sliceTypeAdapt;                                                  // enable slice type adaptation (STA)
+  bool                m_treatAsSubPic;
+
+  double              m_reservedDouble[10];
+
+  // internal state variables
+  bool                m_configDone;                                                      // state variable, Private context used for internal data ( do not change )
+  bool                m_confirmFailed;                                                   // state variable, Private context used for internal data ( do not change )
+  vvencLoggingCallback m_msgFnc;                                                         // logger callback function (internal info/errors will use this function to return log messages)
+  void                *m_msgCtx;                                                         // caller context ( if not needed null )
+
 }vvenc_config;
 
+/* vvenc_config_default
+  This method initializes the vvenc_config parameters to default values (constructor).
+ \param[in]  vvenc_config* pointer to vvenc_config struct that contains encoder parameters
+ \retval     none
+ \pre        None
+*/
 VVENC_DECL void vvenc_config_default( vvenc_config *cfg );
 
-VVENC_DECL int vvenc_init_preset( vvenc_config *cfg, vvencPresetMode preset );
-
+/* vvenc_init_default
+  This method initializes the vvenc_config parameters to default values by using all required
+  parameters size, framerate, bitrate, qp, preset.
+ \param[in]  vvenc_config* pointer to vvenc_config struct that contains encoder parameters
+ \param[in]  width  source width in pixel
+ \param[in]  height source height in pixel
+ \param[in]  framerate source frame-rates (Hz)
+ \param[in]  targetbitrate bitrate in bps (0: use fix qp, >0: enable rate control)
+ \param[in]  qp QP value of key-picture (integer in range 0 to 63, default: 32)
+ \param[in]  preset enum of used preset (default: VVENC_MEDIUM)
+ \retval     int if non-zero an error occurred (see ErrorCodes), otherwise VVENC_OK indicates success.
+ \pre        None
+*/
 VVENC_DECL int vvenc_init_default( vvenc_config *cfg, int width, int height, int framerate, int targetbitrate, int qp, vvencPresetMode preset );
 
+/* vvenc_init_preset
+  This method overwrites encoder parameter by using a preset.
+ \param[in]  vvenc_config* pointer to vvenc_config struct that contains encoder parameters
+ \param[in]  preset enum of used preset (default: VVENC_MEDIUM)
+ \retval     int if non-zero an error occurred (see ErrorCodes), otherwise VVENC_OK indicates success.
+ \pre        vvenc_config_default() or vvenc_init_default() must be called first 
+*/
+VVENC_DECL int vvenc_init_preset( vvenc_config *cfg, vvencPresetMode preset );
+
+
+/* vvenc_set_logging_callback
+ This method registers a log message callback function.
+ This callback is automatically used when calling vvenc_encoder_open().
+ If no such function has been registered, the library will omit all messages.
+ \param[in]  vvenc_config* pointer to vvenc_config struct that contains encoder parameters
+ \param[in]  msgCtx pointer of the caller, if not needed set it to null
+ \param[in]  msgFnc Log message callback function.
+ \retval     none
+*/
+VVENC_DECL void vvenc_set_msg_callback( vvenc_config *cfg, void * msgCtx, vvencLoggingCallback msgFnc );
+
+/* vvenc_init_config_parameter (optional)
+  This method initialize the encoder parameter and sets all parameter the are not initialized yet.
+  All not initialized parameters are set to valid values.
+  Is automatically called in vvenc_encoder_open().
+ \param[in]  vvenc_config* pointer to vvenc_config struct that contains encoder parameters
+  \retval    bool if true an error occurred, otherwise false.
+ \pre        vvenc_config_default() or vvenc_init_default() must be called first 
+*/
 VVENC_DECL bool vvenc_init_config_parameter( vvenc_config *cfg );
 
+
+/* following defines should be used when using the string api interface function
+ * venc_set_param (config, optionname, value)
+*/
+
+//       macro                         mapped option name     vvenc_config parameter name
+#define VVENC_OPT_SIZE                 "size"                 // m_SourceWidthxm_SourceHeight e.g. 1920x1080
+#define VVENC_OPT_WIDTH                "sourcewidth"          // m_SourceWidth  (in pixel)
+#define VVENC_OPT_HEIGHT               "sourceheight"         // m_SourceHeight (in pixel)
+#define VVENC_OPT_FRAMERATE            "framerate"            // m_FrameRate    (fps numerator in Hz)
+#define VVENC_OPT_FRAMESCALE           "framescale"           // m_FrameScale   (fps denominator in Hz)
+#define VVENC_OPT_FPS                  "fps"                  // Framerate as fraction (num/denom)
+#define VVENC_OPT_TICKSPERSEC          "tickspersec"          // m_TicksPerSecond
+#define VVENC_OPT_INPUTBITDEPTH        "inputbitdepth"        // m_inputBitDepth
+#define VVENC_OPT_FRAMES               "framestobeencoded"    // m_framesToBeEncoded
+#define VVENC_OPT_PRESET               "preset"               // set preset like "faster,fast,medium,slow,slower
+#define VVENC_OPT_THREADS              "threads"              // m_numThreads
+#define VVENC_OPT_BITRATE              "bitrate"              // m_RCTargetBitrate
+#define VVENC_OPT_QP                   "qp"                   // m_QP
+#define VVENC_OPT_TILES                "tiles"                // number of tile column x rows in m_numTileColsxm_numTileRows, e.g. 2x2
+#define VVENC_OPT_VERBOSITY            "verbosity"            // m_verbosity
+
+#define VVENC_OPT_PROFILE               "profile"             // m_profile
+#define VVENC_OPT_LEVEL                 "level"               // m_level
+#define VVENC_OPT_TIER                  "tier"                // m_levelTier
+
+#define VVENC_OPT_REFRESHDSEC           "refreshsec"          // m_IntraPeriodSec
+#define VVENC_OPT_INTRAPERIOD           "intraperiod"         // m_IntraPeriod
+#define VVENC_OPT_DECODINGREFRESHTYPE   "decodingrefreshtype" // m_DecodingRefreshType
+#define VVENC_OPT_GOPSIZE               "gopsize"             // m_GOPSize
+
+#define VVENC_OPT_QPA                   "qpa"                 // m_usePerceptQPA
+#define VVENC_OPT_RCPASSES              "passes"              // m_RCNumPasses
+#define VVENC_OPT_RCPASS                "pass"                // m_RCPass
+#define VVENC_OPT_INTERNALBITDEPTH      "internalbitdepth"    // m_internalBitDepth
+#define VVENC_OPT_HDR                   "hdr"                 // m_HdrMode
+#define VVENC_OPT_SEGMENT               "segment"             // m_SegmentMode
+
+/* vvenc_set_param (optional)
+  This method sets one parameter by name.
+  numerical range is not checked until vvenc_init_config_parameter()
+  \param[in]  vvenc_config* pointer to vvenc_config struct that contains encoder parameters
+  \param[in]  name option name as string
+              use "help", "longhelp", "fullhelp" to print available options
+  \param[in]  value option value as string
+              value=NULL means "true" for boolean options, but is a BAD_VALUE for non-booleans.
+  \retval    returns 0 on success, or returns one of the following values:
+             VVENC_BAD_VALUE occurs only if it can't even parse the value,
+             VVENC_PARAM_INFO occurs when a information should be printed (e.g. help, version)
+  \pre       vvenc_config_default() or vvenc_init_default() must be called first 
+*/
+#define VVENC_PARAM_BAD_NAME  (-1)
+#define VVENC_PARAM_BAD_VALUE (-2)
+#define VVENC_PARAM_INFO      (1)
+VVENC_DECL int vvenc_set_param(vvenc_config *cfg, const char *name, const char *value);
+
+/* vvenc_set_param_list (optional)
+  This method sets a list of parameters by name.
+  arguments must be separated as would be used in command line. e.g.: --bitrate 500000 --framerate 50
+  numerical range is not checked until vvenc_init_config_parameter()
+  \param[in]  vvenc_config* pointer to vvenc_config struct that contains encoder parameters
+  \param[in]  argc number or arguments in argv string list
+  \param[in]  argv list of char* (argv[]); option name must be defined by prefix -- or -
+  \retval     returns 0 on success, > 0 if an information was printed (help), -1 on failure.
+  \pre        vvenc_config_default() or vvenc_init_default() must be called first 
+*/
+VVENC_DECL int vvenc_set_param_list(vvenc_config *c, int argc, char* argv[] );
+
+/* vvenc_get_config_as_string (optional)
+  This method returns the encoder configuration as string.
+  \param[in]  vvenc_config* pointer to vvenc_config struct that contains encoder parameters
+  \param[in]  eMsgLevel verbosity level
+  \retval     const char* encoder configuration as string
+  \pre        vvenc_config_default() or vvenc_init_default() must be called first 
+*/
 VVENC_DECL const char* vvenc_get_config_as_string( vvenc_config *cfg, vvencMsgLevel eMsgLevel );
 
 #ifdef __cplusplus
