@@ -1,45 +1,41 @@
 /* -----------------------------------------------------------------------------
-The copyright in this software is being made available under the BSD
+The copyright in this software is being made available under the Clear BSD
 License, included below. No patent rights, trademark rights and/or 
 other Intellectual Property Rights other than the copyrights concerning 
 the Software are granted under this license.
 
-For any license concerning other Intellectual Property rights than the software,
-especially patent licenses, a separate Agreement needs to be closed. 
-For more information please contact:
+The Clear BSD License
 
-Fraunhofer Heinrich Hertz Institute
-Einsteinufer 37
-10587 Berlin, Germany
-www.hhi.fraunhofer.de/vvc
-vvc@hhi.fraunhofer.de
-
-Copyright (c) 2019-2021, Fraunhofer-Gesellschaft zur Förderung der angewandten Forschung e.V.
+Copyright (c) 2019-2023, Fraunhofer-Gesellschaft zur Förderung der angewandten Forschung e.V. & The VVenC Authors.
 All rights reserved.
 
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions are met:
+Redistribution and use in source and binary forms, with or without modification,
+are permitted (subject to the limitations in the disclaimer below) provided that
+the following conditions are met:
 
- * Redistributions of source code must retain the above copyright notice,
-   this list of conditions and the following disclaimer.
- * Redistributions in binary form must reproduce the above copyright notice,
-   this list of conditions and the following disclaimer in the documentation
-   and/or other materials provided with the distribution.
- * Neither the name of Fraunhofer nor the names of its contributors may
-   be used to endorse or promote products derived from this software without
-   specific prior written permission.
+     * Redistributions of source code must retain the above copyright notice,
+     this list of conditions and the following disclaimer.
 
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
-BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
-THE POSSIBILITY OF SUCH DAMAGE.
+     * Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
+
+     * Neither the name of the copyright holder nor the names of its
+     contributors may be used to endorse or promote products derived from this
+     software without specific prior written permission.
+
+NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY
+THIS LICENSE. THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
+CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
+BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
+IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+POSSIBILITY OF SUCH DAMAGE.
 
 
 ------------------------------------------------------------------------------------------- */
@@ -60,12 +56,19 @@ THE POSSIBILITY OF SUCH DAMAGE.
 //! \ingroup CommonLib
 //! \{
 
-namespace vvenc {
+namespace vvenc
+{
+
+struct CtxTpl
+{
+  // lower 5 bits are absSum1, upper 3 bits are numPos
+  uint8_t ctxTpl;
+};
 
 struct CoeffCodingContext
 {
 public:
-  CoeffCodingContext( const TransformUnit& tu, ComponentID component, bool signHide, bool bdpcm = false );
+  CoeffCodingContext( const TransformUnit& tu, ComponentID component, bool signHide, bool bdpcm = false, CtxTpl* tplBuf = nullptr );
 public:
   void  initSubblock    ( int SubsetId, bool sigGroupFlag = false );
 public:
@@ -87,7 +90,6 @@ public:
   unsigned        log2CGWidth     ()                        const { return m_log2CGWidth; }
   unsigned        log2CGHeight    ()                        const { return m_log2CGHeight; }
   unsigned        log2CGSize      ()                        const { return m_log2CGSize; }
-  bool            extPrec         ()                        const { return m_extendedPrecision; }
   int             maxLog2TrDRange ()                        const { return m_maxLog2TrDynamicRange; }
   unsigned        maxNumCoeff     ()                        const { return m_maxNumCoeff; }
   int             scanPosLast     ()                        const { return m_scanPosLast; }
@@ -146,7 +148,6 @@ public:
     }
 #undef UPDATE
 
-
     int ctxOfs = std::min((sumAbs+1)>>1, 3) + ( diag < 2 ? 4 : 0 );
 
     if( m_chType == CH_L )
@@ -159,6 +160,75 @@ public:
     return m_sigFlagCtxSet[std::max( 0, state-1 )]( ctxOfs );
   }
 
+  unsigned sigCtxIdAbsWithAcc( const int scanPos, const int state )
+  {
+    const auto scanEl       = m_scan[scanPos];
+    const uint32_t posY     = scanEl.y;
+    const uint32_t posX     = scanEl.x;
+    const int32_t blkPos    = scanEl.idx;
+    const int      diag     = posX + posY;
+    const int      tplVal   = m_tplBuf[-blkPos].ctxTpl;
+    const int      numPos   = tplVal >> 5u;
+    const int      sumAbs   = tplVal & 31;
+
+    int ctxOfs = std::min( ( sumAbs + 1 ) >> 1, 3 ) + ( diag < 2 ? 4 : 0 );
+
+    if( isLuma( m_chType ) )
+    {
+      ctxOfs += diag < 5 ? 4 : 0;
+    }
+    m_tmplCpDiag = diag;
+    m_tmplCpSum1 = sumAbs - numPos;
+    return m_sigFlagCtxSet[std::max( 0, state-1 )]( ctxOfs );
+  }
+
+  void absVal1stPass( const int scanPos, const TCoeffSig absLevel1 )
+  {
+    CHECKD( !absLevel1, "Shound not be called if '0'!" );
+
+    const auto scanEl     = m_scan[scanPos];
+    const uint32_t posY   = scanEl.y;
+    const uint32_t posX   = scanEl.x;
+    const int32_t  blkPos = scanEl.idx;
+
+    auto update_deps = [&]( int offset )
+    {
+      auto& ctx   = m_tplBuf[-blkPos + offset];
+      ctx.ctxTpl += uint8_t( 32 + absLevel1 );
+    };
+
+    if( posY > 1 ) update_deps( 2 * m_width );
+    if( posY > 0
+     && posX > 0 ) update_deps( m_width + 1 );
+    if( posY > 0 ) update_deps( m_width );
+    if( posX > 1 ) update_deps( 2 );
+    if( posX > 0 ) update_deps( 1 );
+  }
+  
+
+  void remAbsVal1stPass( const int scanPos, const TCoeffSig absLevel1 )
+  {
+    CHECKD( !absLevel1, "Shound not be called if '0'!" );
+
+    const auto scanEl     = m_scan[scanPos];
+    const uint32_t posY   = scanEl.y;
+    const uint32_t posX   = scanEl.x;
+    const int32_t  blkPos = scanEl.idx;
+
+    auto update_deps = [&]( int offset )
+    {
+      auto& ctx   = m_tplBuf[-blkPos + offset];
+      ctx.ctxTpl -= uint8_t( 32 + absLevel1 );
+    };
+
+    if( posY > 1 ) update_deps( 2 * m_width );
+    if( posY > 0
+     && posX > 0 ) update_deps( m_width + 1 );
+    if( posY > 0 ) update_deps( m_width );
+    if( posX > 1 ) update_deps( 2 );
+    if( posX > 0 ) update_deps( 1 );
+  }
+
   uint8_t ctxOffsetAbs()
   {
     int offset = 0;
@@ -380,7 +450,6 @@ private:
   const unsigned            m_log2BlockHeight;
   const unsigned            m_maxNumCoeff;
   const bool                m_signHiding;
-  const bool                m_extendedPrecision;
   const int                 m_maxLog2TrDynamicRange;
   const ScanElement *       m_scan;
   const ScanElement *       m_scanCG;
@@ -415,6 +484,7 @@ private:
   int                       m_remainingContextBins;
   std::bitset<MLS_GRP_NUM>  m_sigCoeffGroupFlag;
   const bool                m_bdpcm;
+  CtxTpl*                   m_tplBuf;
 };
 
 
