@@ -1,45 +1,41 @@
 /* -----------------------------------------------------------------------------
-The copyright in this software is being made available under the BSD
+The copyright in this software is being made available under the Clear BSD
 License, included below. No patent rights, trademark rights and/or 
 other Intellectual Property Rights other than the copyrights concerning 
 the Software are granted under this license.
 
-For any license concerning other Intellectual Property rights than the software,
-especially patent licenses, a separate Agreement needs to be closed. 
-For more information please contact:
+The Clear BSD License
 
-Fraunhofer Heinrich Hertz Institute
-Einsteinufer 37
-10587 Berlin, Germany
-www.hhi.fraunhofer.de/vvc
-vvc@hhi.fraunhofer.de
-
-Copyright (c) 2019-2021, Fraunhofer-Gesellschaft zur Förderung der angewandten Forschung e.V.
+Copyright (c) 2019-2023, Fraunhofer-Gesellschaft zur Förderung der angewandten Forschung e.V. & The VVenC Authors.
 All rights reserved.
 
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions are met:
+Redistribution and use in source and binary forms, with or without modification,
+are permitted (subject to the limitations in the disclaimer below) provided that
+the following conditions are met:
 
- * Redistributions of source code must retain the above copyright notice,
-   this list of conditions and the following disclaimer.
- * Redistributions in binary form must reproduce the above copyright notice,
-   this list of conditions and the following disclaimer in the documentation
-   and/or other materials provided with the distribution.
- * Neither the name of Fraunhofer nor the names of its contributors may
-   be used to endorse or promote products derived from this software without
-   specific prior written permission.
+     * Redistributions of source code must retain the above copyright notice,
+     this list of conditions and the following disclaimer.
 
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
-BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
-THE POSSIBILITY OF SUCH DAMAGE.
+     * Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
+
+     * Neither the name of the copyright holder nor the names of its
+     contributors may be used to endorse or promote products derived from this
+     software without specific prior written permission.
+
+NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY
+THIS LICENSE. THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
+CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
+BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
+IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+POSSIBILITY OF SUCH DAMAGE.
 
 
 ------------------------------------------------------------------------------------------- */
@@ -59,6 +55,11 @@ THE POSSIBILITY OF SUCH DAMAGE.
 #include <stdlib.h>
 #include <memory.h>
 
+#if defined( TARGET_SIMD_X86 )
+#  include "CommonDefX86.h"
+#  include <simde/x86/sse4.1.h>
+#endif
+
 //! \ingroup CommonLib
 //! \{
 
@@ -256,7 +257,6 @@ void QuantRDOQ2::quant( TransformUnit &tu, const ComponentID compID, const CCoef
   const uint32_t uiHeight   = rect.height;
 
   const CCoeffBuf&  piCoef   = pSrc;
-        CoeffSigBuf piQCoef  = tu.getCoeffs(compID);
 
   const bool useTransformSkip = tu.mtsIdx[compID]==MTS_SKIP;
 
@@ -270,11 +270,11 @@ void QuantRDOQ2::quant( TransformUnit &tu, const ComponentID compID, const CCoef
 
   if( useRDOQ )
   {
-    if (!m_useSelectiveRDOQ || xNeedRDOQ(tu, compID, piCoef, cQP))
+    if( !tu.cs->picture->useScSelectiveRdoq || xNeedRDOQ( tu, compID, piCoef, cQP ) )
     {
       if( useTransformSkip )
       {
-        if(tu.cu->bdpcmM[toChannelType(compID)])
+        if( tu.cu->bdpcmM[toChannelType( compID )] )
         {
           forwardRDPCM( tu, compID, pSrc, uiAbsSum, cQP, ctx );
         }
@@ -290,8 +290,7 @@ void QuantRDOQ2::quant( TransformUnit &tu, const ComponentID compID, const CCoef
     }
     else
     {
-      piQCoef.fill(0);
-      uiAbsSum = 0;
+      uiAbsSum    = 0;
       tu.lastPos[compID] = -1;
     }
   }
@@ -301,9 +300,9 @@ void QuantRDOQ2::quant( TransformUnit &tu, const ComponentID compID, const CCoef
   }
 }
 
-inline cost_t QuantRDOQ2::xiGetICost( int iRate ) const
+inline cost_t QuantRDOQ2::xiGetICost(int iRate ) const
 {
-  return m_dLambda * iRate;
+  return (cost_t)(m_dLambda * iRate);
 }
 
 inline cost_t QuantRDOQ2::xGetIEPRate() const
@@ -325,7 +324,6 @@ inline cost_t QuantRDOQ2::xiGetICRateCost( const uint32_t     uiAbsLevel,
                                             const int          remRegBins,
                                             unsigned           goRiceZero,
                                             const uint16_t     ui16AbsGoRice,
-                                            const bool         useLimitedPrefixLength,
                                             const int          maxLog2TrDynamicRange ) const
 {
   cost_t iRate = xGetIEPRate();
@@ -339,22 +337,6 @@ inline cost_t QuantRDOQ2::xiGetICRateCost( const uint32_t     uiAbsLevel,
       length = symbol >> ui16AbsGoRice;
       iRate += ( length + 1 + ui16AbsGoRice ) << SCALE_BITS;
     }
-    else if( useLimitedPrefixLength )
-    {
-      const uint32_t maximumPrefixLength = ( 32 - ( COEF_REMAIN_BIN_REDUCTION + maxLog2TrDynamicRange ) );
-
-      uint32_t prefixLength = 0;
-      uint32_t suffix = ( symbol >> ui16AbsGoRice ) - COEF_REMAIN_BIN_REDUCTION;
-
-      while( ( prefixLength < maximumPrefixLength ) && ( suffix > ( ( 2 << prefixLength ) - 2 ) ) )
-      {
-        prefixLength++;
-      }
-
-      const uint32_t suffixLength = ( prefixLength == maximumPrefixLength ) ? ( maxLog2TrDynamicRange - ui16AbsGoRice ) : ( prefixLength + 1/*separator*/ );
-
-      iRate += ( COEF_REMAIN_BIN_REDUCTION + prefixLength + suffixLength + ui16AbsGoRice ) << SCALE_BITS;
-    }
     else
     {
       length = ui16AbsGoRice;
@@ -368,70 +350,54 @@ inline cost_t QuantRDOQ2::xiGetICRateCost( const uint32_t     uiAbsLevel,
   }
   else
   {
-  const uint32_t cthres = 4;
-  if( uiAbsLevel >= cthres )
-  {
-    uint32_t symbol = ( uiAbsLevel - cthres ) >> 1;
-    uint32_t length;
-    const int threshold = COEF_REMAIN_BIN_REDUCTION;
-    if( symbol < ( threshold << ui16AbsGoRice ) )
+    const uint32_t cthres = 4;
+    if( uiAbsLevel >= cthres )
     {
-      length = symbol >> ui16AbsGoRice;
-      iRate += ( length + 1 + ui16AbsGoRice ) << SCALE_BITS;
-    }
-    else if( useLimitedPrefixLength )
-    {
-      const uint32_t maximumPrefixLength = ( 32 - ( COEF_REMAIN_BIN_REDUCTION + maxLog2TrDynamicRange ) );
-
-      uint32_t prefixLength = 0;
-      uint32_t suffix = ( symbol >> ui16AbsGoRice ) - COEF_REMAIN_BIN_REDUCTION;
-
-      while( ( prefixLength < maximumPrefixLength ) && ( suffix > ( ( 2 << prefixLength ) - 2 ) ) )
+      uint32_t symbol = ( uiAbsLevel - cthres ) >> 1;
+      uint32_t length;
+      const int threshold = COEF_REMAIN_BIN_REDUCTION;
+      if( symbol < ( threshold << ui16AbsGoRice ) )
       {
-        prefixLength++;
+        length = symbol >> ui16AbsGoRice;
+        iRate += ( length + 1 + ui16AbsGoRice ) << SCALE_BITS;
+      }
+      else
+      {
+        length = ui16AbsGoRice;
+        symbol = symbol - ( threshold << ui16AbsGoRice );
+        while( symbol >= ( 1 << length ) )
+        {
+          symbol -= ( 1 << ( length++ ) );
+        }
+        iRate += ( threshold + length + 1 - ui16AbsGoRice + length ) << SCALE_BITS;
       }
 
-      const uint32_t suffixLength = ( prefixLength == maximumPrefixLength ) ? ( maxLog2TrDynamicRange - ui16AbsGoRice ) : ( prefixLength + 1/*separator*/ );
-
-      iRate += ( COEF_REMAIN_BIN_REDUCTION + prefixLength + suffixLength + ui16AbsGoRice ) << SCALE_BITS;
+      iRate += fracBitsGt1.intBits[1];
+      iRate += fracBitsPar.intBits[( uiAbsLevel - 2 ) & 1];
+      iRate += fracBitsGt2.intBits[1];
+    }
+    else if( uiAbsLevel == 1 )
+    {
+      iRate += fracBitsGt1.intBits[0];
+    }
+    else if( uiAbsLevel == 2 )
+    {
+      iRate += fracBitsGt1.intBits[1];
+      iRate += fracBitsPar.intBits[0];
+      iRate += fracBitsGt2.intBits[0];
+    }
+    else if( uiAbsLevel == 3 )
+    {
+      iRate += fracBitsGt1.intBits[1];
+      iRate += fracBitsPar.intBits[1];
+      iRate += fracBitsGt2.intBits[0];
     }
     else
     {
-      length = ui16AbsGoRice;
-      symbol = symbol - ( threshold << ui16AbsGoRice );
-      while( symbol >= ( 1 << length ) )
-      {
-        symbol -= ( 1 << ( length++ ) );
-      }
-      iRate += ( threshold + length + 1 - ui16AbsGoRice + length ) << SCALE_BITS;
+      iRate = 0;
     }
-
-    iRate += fracBitsGt1.intBits[1];
-    iRate += fracBitsPar.intBits[( uiAbsLevel - 2 ) & 1];
-    iRate += fracBitsGt2.intBits[1];
   }
-  else if( uiAbsLevel == 1 )
-  {
-    iRate += fracBitsGt1.intBits[0];
-  }
-  else if( uiAbsLevel == 2 )
-  {
-    iRate += fracBitsGt1.intBits[1];
-    iRate += fracBitsPar.intBits[0];
-    iRate += fracBitsGt2.intBits[0];
-  }
-  else if( uiAbsLevel == 3 )
-  {
-    iRate += fracBitsGt1.intBits[1];
-    iRate += fracBitsPar.intBits[1];
-    iRate += fracBitsGt2.intBits[0];
-  }
-  else
-  {
-    iRate = 0;
-  }
-  }
-  return xiGetICost( iRate );
+  return xiGetICost( (int)iRate );
 }
 
 inline cost_t QuantRDOQ2::xiGetCostSigCoeffGroup( const BinFracBits& fracBitsSigCG, unsigned uiSignificanceCoeffGroup ) const
@@ -509,7 +475,7 @@ static inline cost_t _dist( cost_t iErr, cost_t iErrScale, int64_t iErrScaleShif
 template< bool bSBH, bool bUseScalingList >
 int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &compID, const CCoeffBuf &pSrc, TCoeff &uiAbsSum, const QpParam &cQP, const Ctx &ctx )
 {
-  CoeffCodingContext cctx( tu, compID, bSBH );
+  CoeffCodingContext cctx( tu, compID, bSBH, false, m_tplBuf );
   const FracBitsAccess& fracBits = ctx.getFracBitsAcess();
 
   const SPS &sps            = *tu.cs->sps;
@@ -519,10 +485,9 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
   const ChannelType chType  = toChannelType( compID );
   const int channelBitDepth = sps.bitDepths[ chType ];
 
-  const bool extendedPrecision     = sps.spsRExt.extendedPrecisionProcessing;
   const int  maxLog2TrDynamicRange = sps.getMaxLog2TrDynamicRange(chType);
 
-  if( compID != COMP_Cr )
+  if( compID != COMP_Cr || !tu.cbf[COMP_Cb] )
     xInitLastPosBitsTab( cctx, uiWidth, uiHeight, chType, fracBits );
 
   /* for 422 chroma blocks, the effective scaling applied during transformation is not a power of 2, hence it cannot be
@@ -532,12 +497,7 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
   */
 
   // Represents scaling through forward transform
-  int iTransformShift = getTransformShift(channelBitDepth, rect.size(), maxLog2TrDynamicRange);
-
-  if (tu.mtsIdx[compID]==MTS_SKIP && extendedPrecision)
-  {
-    iTransformShift = std::max<int>(0, iTransformShift);
-  }
+  const int iTransformShift = getTransformShift(channelBitDepth, rect.size(), maxLog2TrDynamicRange);
 
   const uint32_t uiLog2BlockWidth   = Log2(uiWidth);
   const uint32_t uiLog2BlockHeight  = Log2(uiHeight);
@@ -609,9 +569,22 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
   //////////////////////////////////////////////////////////////////////////
   //  Loop over sub-sets (coefficient groups)
   //////////////////////////////////////////////////////////////////////////
+  
+  TCoeff thres = 0, useThres = 0;
+  
+  if( iQBits )
+    thres = TCoeff( ( int64_t( m_thrVal ) << ( iQBits - 1 ) ) );
+  else
+    thres = TCoeff( ( int64_t( m_thrVal >> 1 ) << iQBits ) );
 
+  if( !bUseScalingList )
+  {
+    useThres = thres / ( defaultQuantScale << 2 );
+  }
+
+  const bool scanFirstBlk = !bUseScalingList && log2CGSize == 4 && cctx.log2CGWidth() == 2;
 #if ENABLE_SIMD_OPT_QUANT && defined( TARGET_SIMD_X86 )
-  const bool scanFirstSimd = !bUseScalingList && iScanPos >= 16 && log2CGSize == 4 && cctx.log2CGWidth() == 2 && read_x86_extension_flags() > SCALAR;
+  const bool isSimd       = read_x86_extension_flags() > SCALAR;
 #endif
 
   int subSetId = iScanPos >> log2CGSize;
@@ -627,48 +600,34 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
     {
 #if ENABLE_SIMD_OPT_QUANT && defined( TARGET_SIMD_X86 )
       // if more than one 4x4 coding subblock is available, use SIMD to find first subblock with coefficient larger than threshold
-      if( scanFirstSimd )
+      if( scanFirstBlk && iScanPos >= 16 && isSimd )
       {
         // move the pointer to the beginning of the current subblock
         const int firstTestPos  = iScanPos - iScanPosinCG;
         uint32_t  uiBlkPos      = cctx.blockPos( firstTestPos );
 
-        const __m128i xquant = _mm_set1_epi32( defaultQuantScale );
-        const __m128i xadd   = _mm_set1_epi32( iQOffset );
+        const __m128i xdfTh = _mm_set1_epi32( useThres );
 
-        // read first line of the subblock and check for quantized non-zero coefficients
-        // assumming the subblocks are dense 4x4 blocks in raster scan order with the stride of uiwidth
-        __m128i xl0 = _mm_abs_epi32  ( _mm_loadu_si128( (const __m128i*) & plSrcCoeff[uiBlkPos] ) );
-                xl0 = _mm_mullo_epi32( xl0, xquant );
-                xl0 = _mm_add_epi32  ( xl0, xadd );
-                xl0 = _mm_srai_epi32 ( xl0, iQBits );
-        __m128i xdf = xl0;
+        // read first line of the subblock and check for coefficients larger than the threshold
+        // assumming the subblocks are dense 4x4 blocks in raster scan order with the stride of tuPars.m_width
+        __m128i xl0 = _mm_abs_epi32( _mm_loadu_si128( ( const __m128i* ) &plSrcCoeff[uiBlkPos] ) );
+        __m128i xdf = _mm_cmpgt_epi32( xl0, xdfTh );
 
         // same for the next line in the subblock
         uiBlkPos += uiWidth;
-        xl0 = _mm_abs_epi32  ( _mm_loadu_si128( (const __m128i*) & plSrcCoeff[uiBlkPos] ) );
-        xl0 = _mm_mullo_epi32( xl0, xquant );
-        xl0 = _mm_add_epi32  ( xl0, xadd );
-        xl0 = _mm_srai_epi32 ( xl0, iQBits );
-        xdf = _mm_or_si128   ( xdf, xl0 );
+        xl0 = _mm_abs_epi32( _mm_loadu_si128( ( const __m128i* ) &plSrcCoeff[uiBlkPos] ) );
+        xdf = _mm_or_si128( xdf, _mm_cmpgt_epi32( xl0, xdfTh ) );
 
         // and the third line
         uiBlkPos += uiWidth;
-        xl0 = _mm_abs_epi32  ( _mm_loadu_si128( (const __m128i*) & plSrcCoeff[uiBlkPos] ) );
-        xl0 = _mm_mullo_epi32( xl0, xquant );
-        xl0 = _mm_add_epi32  ( xl0, xadd );
-        xl0 = _mm_srai_epi32 ( xl0, iQBits );
-        xdf = _mm_or_si128   ( xdf, xl0 );
+        xl0 = _mm_abs_epi32( _mm_loadu_si128( ( const __m128i* ) &plSrcCoeff[uiBlkPos] ) );
+        xdf = _mm_or_si128( xdf, _mm_cmpgt_epi32( xl0, xdfTh ) );
 
         // and the last line
         uiBlkPos += uiWidth;
-        xl0 = _mm_abs_epi32  ( _mm_loadu_si128( (const __m128i*) & plSrcCoeff[uiBlkPos] ) );
-        xl0 = _mm_mullo_epi32( xl0, xquant );
-        xl0 = _mm_add_epi32  ( xl0, xadd );
-        xl0 = _mm_srai_epi32 ( xl0, iQBits );
-        xdf = _mm_or_si128   ( xdf, xl0 );
+        xl0 = _mm_abs_epi32( _mm_loadu_si128( ( const __m128i* ) &plSrcCoeff[uiBlkPos] ) );
+        xdf = _mm_or_si128( xdf, _mm_cmpgt_epi32( xl0, xdfTh ) );
 
-        // if none of the coeffcients were non-zero, skip the subblock
         if( _mm_testz_si128( xdf, xdf ) )
         {
           iScanPos    -= iScanPosinCG + 1;
@@ -676,8 +635,26 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
           continue;
         }
       }
-
+      else
 #endif
+      if( scanFirstBlk && iScanPos >= 16 )
+      {
+        bool allSmaller = true;
+
+        for( int xScanPosinCG = iScanPosinCG, xScanPos = iScanPos; allSmaller && xScanPosinCG >= 0; xScanPosinCG--, xScanPos-- )
+        {
+          const uint32_t uiBlkPos = cctx.blockPos( xScanPos );
+          allSmaller &= std::abs( plSrcCoeff[uiBlkPos] ) <= useThres;
+        }
+
+        if( allSmaller )
+        {
+          iScanPos    -= iScanPosinCG + 1;
+          iScanPosinCG = -1;
+          continue;
+        }
+      }
+
     findlast2:
       // Fast loop to find last-pos.
       // No need to add distortion to cost as it would be added to both the coded and uncoded cost
@@ -690,9 +667,10 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
         if( bUseScalingList ){ quantScale = quantScaleList[uiBlkPos]; }
         else{                  quantScale = defaultQuantScale; }
         
-        const uint32_t uiMaxAbsLevel = ( abs( plSrcCoeff[uiBlkPos] ) * quantScale + iQOffset ) >> iQBits;
+        const uint32_t uiMaxAbsLevel = ( std::abs( plSrcCoeff[uiBlkPos] ) * quantScale + iQOffset ) >> iQBits;
 
-        if( uiMaxAbsLevel ){
+        if( uiMaxAbsLevel )
+        {
           iLastScanPos = iScanPos;
           lastSubSetId = subSetId;
           break;
@@ -728,7 +706,7 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
         quantScale = defaultQuantScale;
         iErrScale  = defaultErrScale;
       }
-      const int iScaledLevel = abs( plSrcCoeff[uiBlkPos] ) * quantScale;
+      const int iScaledLevel = std::abs( plSrcCoeff[uiBlkPos] ) * quantScale;
       const int iAbsLevel    = ( iScaledLevel + iQOffset ) >> iQBits;
 
       //============ Set context models ===============
@@ -736,7 +714,7 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
 
       if( iScanPos != iLastScanPos )
       {
-        ctxIdSig = cctx.sigCtxIdAbs( iScanPos, piDstCoeff, 0 );
+        ctxIdSig = cctx.sigCtxIdAbsWithAcc( iScanPos, 0 );
       }
       uint8_t     ctxOffset     = cctx.ctxOffsetAbs();
       uint32_t    uiParCtx      = cctx.parityCtxIdAbs   ( ctxOffset );
@@ -777,8 +755,8 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
           cost_t iErr1        = iScaledLevel - ( (int64_t)1 << iQBits );
           cost_t iDist1       = _dist( iErr1, iErrScale, iErrScaleShift );
           cost_t iRate1       = remRegBins < 4 ? 
-                                 xiGetICRateCost( 1, fracBitsPar, fracBitsGt1, fracBitsGt2, remRegBins, goRiceZero, goRiceParam, extendedPrecision, maxLog2TrDynamicRange ) -
-                                 xiGetICRateCost( 0, fracBitsPar, fracBitsGt1, fracBitsGt2, remRegBins, goRiceZero, goRiceParam, extendedPrecision, maxLog2TrDynamicRange ):
+                                 xiGetICRateCost( 1, fracBitsPar, fracBitsGt1, fracBitsGt2, remRegBins, goRiceZero, goRiceParam, maxLog2TrDynamicRange ) -
+                                 xiGetICRateCost( 0, fracBitsPar, fracBitsGt1, fracBitsGt2, remRegBins, goRiceZero, goRiceParam, maxLog2TrDynamicRange ):
                                  fracBitsGt1.intBits[ 0 ];
 
           cost_t iCost1       = iDist1 + iRate1 + xiGetCostSigCoef( fracBitsSig, 1 );
@@ -793,6 +771,13 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
         //===== coefficient level estimation =====
         const int iFloor = (int)( iScaledLevel >> iQBits );
         const int iCeil  = iFloor + 1;
+
+        if( remRegBins >= 4 && iScanPos != iLastScanPos && iCeil >= 4 )
+        {
+          int  sumAll = cctx.templateAbsSum( iScanPos, piDstCoeff, 4 );
+          goRiceParam = g_auiGoRiceParsCoeff[ sumAll ];
+        }
+
         if( iScanPos == iLastScanPos )
         {
           // =======================             =======================
@@ -807,13 +792,13 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
             // ----------------- LEVEL > 0  ----------------
             cost_t iErrF       = iScaledLevel - (iFloor << iQBits);
             cost_t iDistF      = _dist( iErrF, iErrScale, iErrScaleShift ); //(iErrF*iErrScale) >> iErrScaleShift;
-            iCurrCostF         = iDistF + xiGetICRateCost( iFloor, fracBitsPar, fracBitsGt1, fracBitsGt2, remRegBins, goRiceZero, goRiceParam, extendedPrecision, maxLog2TrDynamicRange );
+            iCurrCostF         = iDistF + xiGetICRateCost( iFloor, fracBitsPar, fracBitsGt1, fracBitsGt2, remRegBins, goRiceZero, goRiceParam, maxLog2TrDynamicRange );
           }
 
           // ----------------- LEVEL + 1 ----------------
           cost_t iErrC         = iScaledLevel - (iCeil << iQBits);
           cost_t iDistC        = _dist( iErrC, iErrScale, iErrScaleShift ); //(iErrC*iErrScale) >> iErrScaleShift;
-          cost_t iCurrCostC    = iDistC + xiGetICRateCost( iCeil, fracBitsPar, fracBitsGt1, fracBitsGt2, remRegBins, goRiceZero, goRiceParam, extendedPrecision, maxLog2TrDynamicRange );
+          cost_t iCurrCostC    = iDistC + xiGetICRateCost( iCeil, fracBitsPar, fracBitsGt1, fracBitsGt2, remRegBins, goRiceZero, goRiceParam, maxLog2TrDynamicRange );
 
           if( iCurrCostC < iCurrCostF )
           {
@@ -868,7 +853,7 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
               // ----------------- LEVEL = 1 ----------------
               cost_t iErrF      = iScaledLevel - ( iFloor << iQBits );
               cost_t iDistF     = _dist( iErrF, iErrScale, iErrScaleShift ); //( iErrF*iErrScale ) >> iErrScaleShift;
-              iCostF            = iDistF + iCostSig1 + xiGetICRateCost( iFloor, fracBitsPar, fracBitsGt1, fracBitsGt2, remRegBins, goRiceZero, goRiceParam, extendedPrecision, maxLog2TrDynamicRange );
+              iCostF            = iDistF + iCostSig1 + xiGetICRateCost( iFloor, fracBitsPar, fracBitsGt1, fracBitsGt2, remRegBins, goRiceZero, goRiceParam, maxLog2TrDynamicRange );
 
               if( iCostF < iBestCost )
               {
@@ -894,7 +879,7 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
             // ----------------- LEVELS = 1, 2 ----------------
             cost_t iErrC         = iScaledLevel - ( iCeil << iQBits );
             cost_t iDistC        = _dist( iErrC, iErrScale, iErrScaleShift ); //( iErrC*iErrScale ) >> iErrScaleShift;
-            cost_t iCostC        = iDistC + iCostSig1 + xiGetICRateCost( iCeil, fracBitsPar, fracBitsGt1, fracBitsGt2, remRegBins, goRiceZero, goRiceParam, extendedPrecision, maxLog2TrDynamicRange );
+            cost_t iCostC        = iDistC + iCostSig1 + xiGetICRateCost( iCeil, fracBitsPar, fracBitsGt1, fracBitsGt2, remRegBins, goRiceZero, goRiceParam, maxLog2TrDynamicRange );
 
             if( iCostC < iBestCost )
             {
@@ -923,11 +908,11 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
             // ----------------- LEVEL X, X+1 ----------------
             cost_t iErrF        = iScaledLevel - (iFloor << iQBits);
             cost_t iDistF       = _dist( iErrF, iErrScale, iErrScaleShift ); //(iErrF*iErrScale) >> iErrScaleShift;
-            cost_t iCostF       = iDistF + iCostSig1 + xiGetICRateCost( iFloor, fracBitsPar, fracBitsGt1, fracBitsGt2, remRegBins, goRiceZero, goRiceParam, extendedPrecision, maxLog2TrDynamicRange );
+            cost_t iCostF       = iDistF + iCostSig1 + xiGetICRateCost( iFloor, fracBitsPar, fracBitsGt1, fracBitsGt2, remRegBins, goRiceZero, goRiceParam, maxLog2TrDynamicRange );
 
             cost_t iErrC        = iScaledLevel - ( iCeil << iQBits );
             cost_t iDistC       = _dist( iErrC, iErrScale, iErrScaleShift ); //( iErrC*iErrScale ) >> iErrScaleShift;
-            cost_t iCostC       = iDistC + iCostSig1 + xiGetICRateCost( iCeil, fracBitsPar, fracBitsGt1, fracBitsGt2, remRegBins, goRiceZero, goRiceParam, extendedPrecision, maxLog2TrDynamicRange );
+            cost_t iCostC       = iDistC + iCostSig1 + xiGetICRateCost( iCeil, fracBitsPar, fracBitsGt1, fracBitsGt2, remRegBins, goRiceZero, goRiceParam, maxLog2TrDynamicRange );
 
             piCostSig[iScanPosinCG] = iCostSig1;
             if( iCostC < iCostF )
@@ -960,7 +945,8 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
         {
           uiAbsSumCG    += uiLevel;
           iNZbeforePos0 += iScanPosinCG; // hack-> just add instead of checking iScanPosinCG >0 and increment
-          cctx.setSigGroup();          
+          cctx.absVal1stPass( iScanPos, std::min<TCoeff>( 4 + ( uiLevel & 1 ), uiLevel ) );
+          cctx.setSigGroup();
         }
       }
 
@@ -971,8 +957,6 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
       }
       else if( remRegBins >= 4 )
       {
-        int  sumAll = cctx.templateAbsSum(iScanPos, piDstCoeff, 4);
-        goRiceParam = g_auiGoRiceParsCoeff[sumAll];
         remRegBins -= (uiLevel < 2 ? uiLevel : 3) + (iScanPos != iLastScanPos);
       }
 
@@ -1024,7 +1008,12 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
               {
                 int iScanPosTmp = subSetId * iCGSize + iScanPosinCG;
                 uint32_t uiBlkPos = cctx.blockPos( iScanPosTmp );
-                piDstCoeff[uiBlkPos] = 0;
+                if( piDstCoeff[uiBlkPos] )
+                {
+                  int absLevel = std::abs( piDstCoeff[uiBlkPos] );
+                  cctx.remAbsVal1stPass( iScanPosTmp, std::min( absLevel, 4 + ( absLevel & 1 ) ) );
+                  piDstCoeff[uiBlkPos] = 0;
+                }
               }
               uiAbsSumCG = 0;
               if( lastSubSetId == subSetId ) {
@@ -1090,7 +1079,13 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
         } //end for
         for( int iScanPosTmp = bestLastIdxP1; iScanPosTmp <= iLastScanPos; iScanPosTmp++ )
         {
-          piDstCoeff[cctx.blockPos( iScanPosTmp )] = 0;
+          const int uiBlkPos = cctx.blockPos( iScanPosTmp );
+          if( piDstCoeff[uiBlkPos] )
+          {
+            int absLevel = std::abs( piDstCoeff[uiBlkPos] );
+            cctx.remAbsVal1stPass( iScanPosTmp, std::min( absLevel, 4 + ( absLevel & 1 ) ) );
+            piDstCoeff[uiBlkPos] = 0;
+          }
         }
         iLastScanPos = bestLastIdxP1 - 1;
       }
@@ -1128,7 +1123,7 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
         }
         if( iLastNZPosInCG - iFirstNZPosInCG >= SBH_THRESHOLD )
         {
-          iCodedCostCG -= xiGetICost( xGetIEPRate() ); //subtract cost for one sign bin
+          iCodedCostCG -= xiGetICost( (int)xGetIEPRate() ); //subtract cost for one sign bin
           bool bSign    = plSrcCoeff[ cctx.blockPos( iSubPos + iFirstNZPosInCG) ] < 0;
 
           if( bSign != ( uiAbsSumCG & 0x1 ) ) {
@@ -1156,7 +1151,11 @@ int QuantRDOQ2::xRateDistOptQuantFast( TransformUnit &tu, const ComponentID &com
                 iMinCostPos   = n;
               }
             }
+            const int oldAbsVal = std::abs( piDstCoeff[cctx.blockPos( iMinCostPos + iSubPos )] );
+            if( oldAbsVal ) cctx.remAbsVal1stPass( iMinCostPos + iSubPos, std::min( oldAbsVal, 4 + ( oldAbsVal & 1 ) ) );
             piDstCoeff[ cctx.blockPos( iMinCostPos + iSubPos ) ] += piAddSBH[iMinCostPos];
+            const int absVal = std::abs( piDstCoeff[cctx.blockPos( iMinCostPos + iSubPos )] );
+            if( absVal ) cctx.absVal1stPass( iMinCostPos + iSubPos, std::min( absVal, 4 + ( absVal & 1 ) ) );
             uiAbsSumCG   += piAddSBH[iMinCostPos];
             iCodedCostCG += iMinCostDelta;
           }
