@@ -1,45 +1,41 @@
 /* -----------------------------------------------------------------------------
-The copyright in this software is being made available under the BSD
+The copyright in this software is being made available under the Clear BSD
 License, included below. No patent rights, trademark rights and/or 
 other Intellectual Property Rights other than the copyrights concerning 
 the Software are granted under this license.
 
-For any license concerning other Intellectual Property rights than the software,
-especially patent licenses, a separate Agreement needs to be closed. 
-For more information please contact:
+The Clear BSD License
 
-Fraunhofer Heinrich Hertz Institute
-Einsteinufer 37
-10587 Berlin, Germany
-www.hhi.fraunhofer.de/vvc
-vvc@hhi.fraunhofer.de
-
-Copyright (c) 2019-2021, Fraunhofer-Gesellschaft zur Förderung der angewandten Forschung e.V.
+Copyright (c) 2019-2023, Fraunhofer-Gesellschaft zur Förderung der angewandten Forschung e.V. & The VVenC Authors.
 All rights reserved.
 
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions are met:
+Redistribution and use in source and binary forms, with or without modification,
+are permitted (subject to the limitations in the disclaimer below) provided that
+the following conditions are met:
 
- * Redistributions of source code must retain the above copyright notice,
-   this list of conditions and the following disclaimer.
- * Redistributions in binary form must reproduce the above copyright notice,
-   this list of conditions and the following disclaimer in the documentation
-   and/or other materials provided with the distribution.
- * Neither the name of Fraunhofer nor the names of its contributors may
-   be used to endorse or promote products derived from this software without
-   specific prior written permission.
+     * Redistributions of source code must retain the above copyright notice,
+     this list of conditions and the following disclaimer.
 
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
-BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
-THE POSSIBILITY OF SUCH DAMAGE.
+     * Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
+
+     * Neither the name of the copyright holder nor the names of its
+     contributors may be used to endorse or promote products derived from this
+     software without specific prior written permission.
+
+NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY
+THIS LICENSE. THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
+CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
+BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
+IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+POSSIBILITY OF SUCH DAMAGE.
 
 
 ------------------------------------------------------------------------------------------- */
@@ -156,7 +152,7 @@ UnitArea::UnitArea(const ChromaFormat _chromaFormat, const Area& _area) : chroma
 
   for (uint32_t i = 0; i < numCh; i++)
   {
-    blocks[i] = CompArea(ComponentID(i), chromaFormat, _area, true);
+    new (&blocks[i]) CompArea(ComponentID(i), chromaFormat, _area, true);
   }
 }
 
@@ -361,7 +357,6 @@ void CodingUnit::initPuData()
   geoSplitDir       = MAX_UCHAR;
   geoMergeIdx0      = MAX_UCHAR;
   geoMergeIdx1      = MAX_UCHAR;
-  bv                . setZero();
 
   mcControl         = 0;
 
@@ -415,7 +410,6 @@ CodingUnit& CodingUnit::operator=( const InterPredictionData& other )
   interDir          = other.interDir;
   mergeType         = other.mergeType;
   mvRefine          = other.mvRefine;
-  bv                = other.bv;
 
   if( other.mergeFlag && mvdL0SubPu )
   {
@@ -441,11 +435,11 @@ CodingUnit& CodingUnit::operator=( const InterPredictionData& other )
 
 CodingUnit& CodingUnit::operator=( const MotionInfo& mi )
 {
-  interDir = mi.interDir;
+  interDir = mi.interDir();
 
   for( uint32_t i = 0; i < NUM_REF_PIC_LIST_01; i++ )
   {
-    refIdx[i] = mi.refIdx[i];
+    refIdx[i] = mi.miRefIdx[i];
     mv [i][0] = mi.mv[i];
   }
 
@@ -522,7 +516,7 @@ void TransformUnit::init(TCoeffSig** coeffs)
   }
 }
 
-TransformUnit& TransformUnit::operator=(const TransformUnit& other)
+TransformUnit& TransformUnit::operator=( const TransformUnit& other )
 {
   CHECK( chromaFormat != other.chromaFormat, "Incompatible formats" );
 
@@ -531,13 +525,13 @@ TransformUnit& TransformUnit::operator=(const TransformUnit& other)
   {
     CHECKD( blocks[i].area() != other.blocks[i].area(), "Transformation units cover different areas" );
 
-    uint32_t area = blocks[i].area();
-
-    if (m_coeffs[i] && other.m_coeffs[i] && m_coeffs[i] != other.m_coeffs[i])
-    {
-      memcpy(m_coeffs[i], other.m_coeffs[i], sizeof(TCoeffSig) * area);
-    }
     cbf[i]      = other.cbf[i];
+    bool cpyRsi = other.cbf[i] || ( i && other.jointCbCr && numBlocks > 1 && ( TU::getCbf( other, COMP_Cb ) || TU::getCbf( other, COMP_Cr ) ) );
+    if( m_coeffs[i] && other.m_coeffs[i] && m_coeffs[i] != other.m_coeffs[i] && cpyRsi )
+    {
+      uint32_t area = blocks[i].area();
+      memcpy( m_coeffs[i], other.m_coeffs[i], sizeof( TCoeffSig ) * area );
+    }
     mtsIdx[i]   = other.mtsIdx[i];
     lastPos[i]  = other.lastPos[i];
   }
@@ -547,17 +541,16 @@ TransformUnit& TransformUnit::operator=(const TransformUnit& other)
   return *this;
 }
 
-void TransformUnit::copyComponentFrom(const TransformUnit& other, const ComponentID i)
+void TransformUnit::copyComponentFrom( const TransformUnit& other, const ComponentID i )
 {
   CHECK( chromaFormat != other.chromaFormat, "Incompatible formats" );
-
   CHECKD( blocks[i].area() != other.blocks[i].area(), "Transformation units cover different areas" );
 
-  uint32_t area = blocks[i].area();
-
-  if (m_coeffs[i] && other.m_coeffs[i] && m_coeffs[i] != other.m_coeffs[i])
+  bool cpyRsi = other.cbf[i] || ( i && other.jointCbCr && blocks.size() > 1 && ( TU::getCbf( other, COMP_Cb ) || TU::getCbf( other, COMP_Cr ) ) );
+  if( m_coeffs[i] && other.m_coeffs[i] && m_coeffs[i] != other.m_coeffs[i] && cpyRsi )
   {
-    memcpy(m_coeffs[i], other.m_coeffs[i], sizeof(TCoeffSig) * area);
+    uint32_t area = blocks[i].area();
+    memcpy( m_coeffs[i], other.m_coeffs[i], sizeof( TCoeffSig ) * area );
   }
 
   cbf[i]      = other.cbf[i];
