@@ -1,46 +1,41 @@
 /* -----------------------------------------------------------------------------
-The copyright in this software is being made available under the BSD
+The copyright in this software is being made available under the Clear BSD
 License, included below. No patent rights, trademark rights and/or 
 other Intellectual Property Rights other than the copyrights concerning 
 the Software are granted under this license.
 
-For any license concerning other Intellectual Property rights than the software,
-especially patent licenses, a separate Agreement needs to be closed. 
-For more information please contact:
+The Clear BSD License
 
-Fraunhofer Heinrich Hertz Institute
-Einsteinufer 37
-10587 Berlin, Germany
-www.hhi.fraunhofer.de/vvc
-vvc@hhi.fraunhofer.de
-
-Copyright (c) 2019-2021, Fraunhofer-Gesellschaft zur Förderung der angewandten Forschung e.V.
+Copyright (c) 2019-2023, Fraunhofer-Gesellschaft zur Förderung der angewandten Forschung e.V. & The VVenC Authors.
 All rights reserved.
 
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions are met:
+Redistribution and use in source and binary forms, with or without modification,
+are permitted (subject to the limitations in the disclaimer below) provided that
+the following conditions are met:
 
- * Redistributions of source code must retain the above copyright notice,
-   this list of conditions and the following disclaimer.
- * Redistributions in binary form must reproduce the above copyright notice,
-   this list of conditions and the following disclaimer in the documentation
-   and/or other materials provided with the distribution.
- * Neither the name of Fraunhofer nor the names of its contributors may
-   be used to endorse or promote products derived from this software without
-   specific prior written permission.
+     * Redistributions of source code must retain the above copyright notice,
+     this list of conditions and the following disclaimer.
 
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
-BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
-THE POSSIBILITY OF SUCH DAMAGE.
+     * Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the distribution.
 
+     * Neither the name of the copyright holder nor the names of its
+     contributors may be used to endorse or promote products derived from this
+     software without specific prior written permission.
+
+NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY
+THIS LICENSE. THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
+CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
+BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
+IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+POSSIBILITY OF SUCH DAMAGE.
 
 ------------------------------------------------------------------------------------------- */
 
@@ -94,6 +89,7 @@ void  EncReshape::init( const VVEncCfg& encCfg )
   {
     m_lumaBD = encCfg.m_internalBitDepth[ CH_L ];
     m_reshapeLUTSize = 1 << m_lumaBD;
+    m_rspBaseQP = encCfg.m_QP;
     m_initCWAnalyze = m_reshapeLUTSize / PIC_ANALYZE_CW_BINS;
     m_initCW = m_reshapeLUTSize / PIC_CODE_CW_BINS;
 
@@ -337,14 +333,21 @@ void EncReshape::calcSeqStats(Picture& pic, SeqInfo &stats)
         sum = leftSum;
         sumSq = leftSumSq;
       }
-
+      
       double average = double(sum) / numPixInPart;
       double variance = double(sumSq) / numPixInPart - average * average;
       int binLen = m_reshapeLUTSize / m_binNum;
       uint32_t binIdx = (uint32_t)(pxlY / binLen);
-      average = average / (double)(1 << (m_lumaBD - 10));
-      variance = variance / (double)(1 << (2 * (m_lumaBD - 10)));
-      binIdx = (uint32_t)((pxlY >> (m_lumaBD - 10)) / binLen);
+      if (m_lumaBD > 10)
+      {
+        average = average / (double)(1 << (m_lumaBD - 10));
+        variance = variance / (double)(1 << (2 * m_lumaBD - 20));
+      }
+      else if (m_lumaBD < 10)
+      {
+        average = average * (double)(1 << (10 - m_lumaBD));
+        variance = variance * (double)(1 << (20 - 2 * m_lumaBD));
+      }
       double varLog10 = log10(variance + 1.0);
       stats.binVar[binIdx] += varLog10;
       binCnt[binIdx]++;
@@ -389,11 +392,6 @@ void EncReshape::calcSeqStats(Picture& pic, SeqInfo &stats)
   }
 
   picY = pic.getOrigBuf(COMP_Y);
-  CPelBuf picU = pic.getOrigBuf(COMP_Cb);
-  CPelBuf picV = pic.getOrigBuf(COMP_Cr);
-  const int widthC = picU.width;
-  const int heightC = picU.height;
-  const int strideC = picU.stride;
   double avgY = 0.0, avgU = 0.0, avgV = 0.0;
   double varY = 0.0, varU = 0.0, varV = 0.0;
   for (int y = 0; y < height; y++)
@@ -405,28 +403,37 @@ void EncReshape::calcSeqStats(Picture& pic, SeqInfo &stats)
     }
     picY.buf += stride;
   }
-  for (int y = 0; y < heightC; y++)
+
+  if( pic.chromaFormat != VVENC_CHROMA_400 )
   {
-    for (int x = 0; x < widthC; x++)
+    CPelBuf picU = pic.getOrigBuf(COMP_Cb);
+    CPelBuf picV = pic.getOrigBuf(COMP_Cr);
+    const int widthC = picU.width;
+    const int heightC = picU.height;
+    const int strideC = picU.stride;
+    for (int y = 0; y < heightC; y++)
     {
-      avgU += picU.buf[x];
-      avgV += picV.buf[x];
-      varU += (int64_t)picU.buf[x] * (int64_t)picU.buf[x];
-      varV += (int64_t)picV.buf[x] * (int64_t)picV.buf[x];
+      for (int x = 0; x < widthC; x++)
+      {
+        avgU += picU.buf[x];
+        avgV += picV.buf[x];
+        varU += (int64_t)picU.buf[x] * (int64_t)picU.buf[x];
+        varV += (int64_t)picV.buf[x] * (int64_t)picV.buf[x];
+      }
+      picU.buf += strideC;
+      picV.buf += strideC;
+    }
+    avgY = avgY / (width * height);
+    avgU = avgU / (widthC * heightC);
+    avgV = avgV / (widthC * heightC);
+    varY = varY / (width * height) - avgY * avgY;
+    varU = varU / (widthC * heightC) - avgU * avgU;
+    varV = varV / (widthC * heightC) - avgV * avgV;
+    if (varY > 0)
+    {
+      stats.ratioStdU = sqrt(varU) / sqrt(varY);
+      stats.ratioStdV = sqrt(varV) / sqrt(varY);
     }
-    picU.buf += strideC;
-    picV.buf += strideC;
-  }
-  avgY = avgY / (width * height);
-  avgU = avgU / (widthC * heightC);
-  avgV = avgV / (widthC * heightC);
-  varY = varY / (width * height) - avgY * avgY;
-  varU = varU / (widthC * heightC) - avgU * avgU;
-  varV = varV / (widthC * heightC) - avgV * avgV;
-  if (varY > 0)
-  {
-    stats.ratioStdU = sqrt(varU) / sqrt(varY);
-    stats.ratioStdV = sqrt(varV) / sqrt(varY);
   }
 }
 
@@ -523,7 +530,7 @@ void EncReshape::preAnalyzerLMCS(Picture& pic, const uint32_t signalType, const
         }
       }
 
-      if (m_rateAdpMode == 2 && reshapeCW.rspBaseQP <= 22) { intraAdp = false; interAdp = false; }
+      if (m_rateAdpMode == 2 && m_rspBaseQP <= 22) { intraAdp = false; interAdp = false; }
       m_sliceReshapeInfo.sliceReshaperEnabled = intraAdp;
       if (!intraAdp && !interAdp)
       {
@@ -532,7 +539,7 @@ void EncReshape::preAnalyzerLMCS(Picture& pic, const uint32_t signalType, const
         return;
       }
 
-      if (m_rateAdpMode == 1 && reshapeCW.rspBaseQP <= 22)
+      if (m_rateAdpMode == 1 && m_rspBaseQP <= 22)
       {
         for (int i = 0; i < m_binNum; i++)
         {
@@ -575,6 +582,99 @@ void EncReshape::preAnalyzerLMCS(Picture& pic, const uint32_t signalType, const
       const int cTid = m_reshapeCW.rspTid;
       bool enableRsp = m_tcase == 5 ? false : (m_tcase < 5 ? (cTid < m_tcase + 1 ? false : true) : (cTid <= 10 - m_tcase ? true : false));
       m_sliceReshapeInfo.sliceReshaperEnabled = enableRsp;
+
+      if( m_sliceReshapeInfo.sliceReshaperEnabled )
+      {
+        m_binNum = PIC_CODE_CW_BINS;
+        CPelBuf picY = pic.getOrigBuf( COMP_Y );
+        const int width = picY.width;
+        const int height = picY.height;
+        const int stride = picY.stride;
+        uint32_t binCnt[PIC_CODE_CW_BINS];
+        std::fill_n( binCnt, m_binNum, 0 );
+
+        m_srcSeqStats = SeqInfo();
+        for( uint32_t y = 0; y < height; y++ )
+        {
+          for( uint32_t x = 0; x < width; x++ )
+          {
+            const Pel pxlY = picY.buf[x];
+            int binLen = m_reshapeLUTSize / m_binNum;
+            uint32_t binIdx = ( uint32_t ) (pxlY / binLen);
+            binCnt[binIdx]++;
+          }
+          picY.buf += stride;
+        }
+
+        for( int b = 0; b < m_binNum; b++ )
+        {
+          m_srcSeqStats.binHist[b] = ( double ) binCnt[b] / ( double ) (m_reshapeCW.rspPicSize);
+        }
+
+        if( m_srcSeqStats.binHist[m_binNum - 1] > 0.0003 )
+        {
+          m_sliceReshapeInfo.sliceReshaperEnabled = false;
+        }
+        if( m_srcSeqStats.binHist[0] > 0.03 )
+        {
+          m_sliceReshapeInfo.sliceReshaperEnabled = false;
+        }
+
+        if( m_sliceReshapeInfo.sliceReshaperEnabled )
+        {
+          double avgY = 0.0;
+          double varY = 0.0;
+          picY = pic.getOrigBuf( COMP_Y );
+          for( int y = 0; y < height; y++ )
+          {
+            for( int x = 0; x < width; x++ )
+            {
+              avgY += picY.buf[x];
+              varY += ( double ) picY.buf[x] * ( double ) picY.buf[x];
+            }
+            picY.buf += stride;
+          }
+          avgY = avgY / (width * height);
+          varY = varY / (width * height) - avgY * avgY;
+
+          if( isChromaEnabled( pic.chromaFormat ) )
+          {
+            CPelBuf picU = pic.getOrigBuf( COMP_Cb );
+            CPelBuf picV = pic.getOrigBuf( COMP_Cr );
+            const int widthC = picU.width;
+            const int heightC = picU.height;
+            const int strideC = picU.stride;
+            double avgU = 0.0, avgV = 0.0;
+            double varU = 0.0, varV = 0.0;
+            for( int y = 0; y < heightC; y++ )
+            {
+              for( int x = 0; x < widthC; x++ )
+              {
+                avgU += picU.buf[x];
+                avgV += picV.buf[x];
+                varU += ( int64_t ) picU.buf[x] * ( int64_t ) picU.buf[x];
+                varV += ( int64_t ) picV.buf[x] * ( int64_t ) picV.buf[x];
+              }
+              picU.buf += strideC;
+              picV.buf += strideC;
+            }
+            avgU = avgU / (widthC * heightC);
+            avgV = avgV / (widthC * heightC);
+            varU = varU / (widthC * heightC) - avgU * avgU;
+            varV = varV / (widthC * heightC) - avgV * avgV;
+            if( varY > 0 )
+            {
+              m_srcSeqStats.ratioStdU = sqrt( varU ) / sqrt( varY );
+              m_srcSeqStats.ratioStdV = sqrt( varV ) / sqrt( varY );
+            }
+          }
+        }
+
+        if( (m_srcSeqStats.ratioStdU + m_srcSeqStats.ratioStdV) > 1.5 && m_srcSeqStats.binHist[1] > 0.5 )
+        {
+          m_sliceReshapeInfo.sliceReshaperEnabled = false;
+        }
+      }
     }
   }
 }
@@ -976,7 +1076,7 @@ void EncReshape::initLUTfromdQPModel()
     absDeltaCW = (deltaCW < 0) ? (-deltaCW) : deltaCW;
     if (absDeltaCW > maxAbsDeltaCW)     {      maxAbsDeltaCW = absDeltaCW;    }
   }
-  m_sliceReshapeInfo.maxNbitsNeededDeltaCW = std::max(1, 1 + floorLog2(maxAbsDeltaCW));
+  m_sliceReshapeInfo.maxNbitsNeededDeltaCW = 1 + floorLog2(std::max(1, maxAbsDeltaCW));
 
   for (int i = 0; i < pwlFwdLUTsize; i++)
   {
@@ -1060,7 +1160,7 @@ void EncReshape::constructReshaperLMCS()
     absDeltaCW = (deltaCW < 0) ? (-deltaCW) : deltaCW;
     if (absDeltaCW > maxAbsDeltaCW) { maxAbsDeltaCW = absDeltaCW; }
   }
-  m_sliceReshapeInfo.maxNbitsNeededDeltaCW = std::max(1, 1 + floorLog2(maxAbsDeltaCW));
+  m_sliceReshapeInfo.maxNbitsNeededDeltaCW = 1 + floorLog2(std::max(1, maxAbsDeltaCW));
 
   histLenth = m_initCW;
   log2HistLenth = floorLog2(histLenth);
@@ -1106,7 +1206,7 @@ void EncReshape::adjustLmcsPivot()
   int bdShift = m_lumaBD - 10;
   int totCW = bdShift != 0 ? (bdShift > 0 ? m_reshapeLUTSize / (1 << bdShift) : m_reshapeLUTSize * (1 << (-bdShift))) : m_reshapeLUTSize;
   int orgCW = totCW / PIC_CODE_CW_BINS;
-  int log2SegSize = Log2(LMCS_SEG_NUM);
+  int log2SegSize = m_lumaBD - floorLog2(LMCS_SEG_NUM);
 
   m_reshapePivot[0] = 0;
   for (int i = 0; i < PIC_CODE_CW_BINS; i++)
